@@ -26,7 +26,12 @@
 #include <QSslCipher>
 #include <QSslError>
 #include <QSharedPointer>
-#include "utility.h"
+
+#ifndef TOKEN_AUTH_ONLY
+#include <QPixmap>
+#endif
+
+#include "common/utility.h"
 #include <memory>
 #include "capabilities.h"
 
@@ -42,16 +47,18 @@ class Account;
 typedef QSharedPointer<Account> AccountPtr;
 class QuotaInfo;
 class AccessManager;
+class SimpleNetworkJob;
 
 
 /**
  * @brief Reimplement this to handle SSL errors from libsync
  * @ingroup libsync
  */
-class AbstractSslErrorHandler {
+class AbstractSslErrorHandler
+{
 public:
     virtual ~AbstractSslErrorHandler() {}
-    virtual bool handleErrors(QList<QSslError>, const QSslConfiguration &conf, QList<QSslCertificate>*, AccountPtr) = 0;
+    virtual bool handleErrors(QList<QSslError>, const QSslConfiguration &conf, QList<QSslCertificate> *, AccountPtr) = 0;
 };
 
 /**
@@ -61,11 +68,17 @@ public:
  * The Account has a name and url. It also has information about credentials,
  * SSL errors and certificates.
  */
-class OWNCLOUDSYNC_EXPORT Account : public QObject {
+class OWNCLOUDSYNC_EXPORT Account : public QObject
+{
     Q_OBJECT
+    Q_PROPERTY(QString id MEMBER _id)
+    Q_PROPERTY(QString davUser MEMBER _davUser)
+    Q_PROPERTY(QString displayName MEMBER _displayName)
+    Q_PROPERTY(QUrl url MEMBER _url)
+
 public:
     static AccountPtr create();
-    ~Account();
+    ~Account() override;
 
     AccountPtr sharedFromThis();
 
@@ -78,6 +91,14 @@ public:
     QString davUser() const;
     void setDavUser(const QString &newDavUser);
 
+    QString davDisplayName() const;
+    void setDavDisplayName(const QString &newDisplayName);
+
+#ifndef TOKEN_AUTH_ONLY
+    QPixmap avatar() const;
+    void setAvatar(const QPixmap &img);
+#endif
+
     /// The name of the account as shown in the toolbar
     QString displayName() const;
 
@@ -88,28 +109,50 @@ public:
     void setUrl(const QUrl &url);
     QUrl url() const { return _url; }
 
+    /// Adjusts _userVisibleUrl once the host to use is discovered.
+    void setUserVisibleHost(const QString &host);
+
     /**
      * @brief The possibly themed dav path for the account. It has
      *        a trailing slash.
      * @returns the (themeable) dav path for the account.
      */
     QString davPath() const;
-    void setDavPath(const QString&s) { _davPath = s; }
-    void setNonShib(bool nonShib);
+    void setDavPath(const QString &s) { _davPath = s; }
 
     /** Returns webdav entry URL, based on url() */
     QUrl davUrl() const;
 
+    /** Returns the legacy permalink url for a file.
+     *
+     * This uses the old way of manually building the url. New code should
+     * use the "privatelink" property accessible via PROPFIND.
+     */
+    QUrl deprecatedPrivateLinkUrl(const QByteArray &numericFileId) const;
+
     /** Holds the accounts credentials */
-    AbstractCredentials* credentials() const;
+    AbstractCredentials *credentials() const;
     void setCredentials(AbstractCredentials *cred);
 
+    /** Create a network request on the account's QNAM.
+     *
+     * Network requests in AbstractNetworkJobs are created through
+     * this function. Other places should prefer to use jobs or
+     * sendRequest().
+     */
+    QNetworkReply *sendRawRequest(const QByteArray &verb,
+        const QUrl &url,
+        QNetworkRequest req = QNetworkRequest(),
+        QIODevice *data = nullptr);
 
-    // For creating various network requests
-    QNetworkReply* sendRequest(const QByteArray &verb,
-                               const QUrl &url,
-                               QNetworkRequest req = QNetworkRequest(),
-                               QIODevice *data = 0);
+    /** Create and start network job for a simple one-off request.
+     *
+     * More complicated requests typically create their own job types.
+     */
+    SimpleNetworkJob *sendRequest(const QByteArray &verb,
+        const QUrl &url,
+        QNetworkRequest req = QNetworkRequest(),
+        QIODevice *data = nullptr);
 
     /** The ssl configuration during the first connection */
     QSslConfiguration getOrCreateSslConfig();
@@ -135,8 +178,8 @@ public:
     void setSslErrorHandler(AbstractSslErrorHandler *handler);
 
     // To be called by credentials only, for storing username and the like
-    QVariant credentialSetting(const QString& key) const;
-    void setCredentialSetting(const QString& key, const QVariant &value);
+    QVariant credentialSetting(const QString &key) const;
+    void setCredentialSetting(const QString &key, const QVariant &value);
 
     /** Assign a client certificate */
     void setCertificate(const QByteArray certficate = QByteArray(), const QString privateKey = QString());
@@ -145,9 +188,21 @@ public:
     const Capabilities &capabilities() const;
     void setCapabilities(const QVariantMap &caps);
 
-    /** Access the server version */
+    /** Access the server version
+     *
+     * For servers >= 10.0.0, this can be the empty string until capabilities
+     * have been received.
+     */
     QString serverVersion() const;
+
+    /** Server version for easy comparison.
+     *
+     * Example: serverVersionInt() >= makeServerVersion(11, 2, 3)
+     *
+     * Will be 0 if the version is not available yet.
+     */
     int serverVersionInt() const;
+
     static int makeServerVersion(int majorVersion, int minorVersion, int patchVersion);
     void setServerVersion(const QString &version);
 
@@ -163,20 +218,26 @@ public:
      */
     bool serverVersionUnsupported() const;
 
-    // Fixed from 8.1 https://github.com/owncloud/client/issues/3730
-    /** Detects a specific bug in older server versions */
-    bool rootEtagChangesNotOnlySubFolderEtags();
+    /** True when the server connection is using HTTP2  */
+    bool isHttp2Supported() { return _http2Supported; }
+    void setHttp2Supported(bool value) { _http2Supported = value; }
 
     void clearCookieJar();
     void lendCookieJarTo(QNetworkAccessManager *guest);
     QString cookieJarPath();
 
     void resetNetworkAccessManager();
-    QNetworkAccessManager* networkAccessManager();
+    QNetworkAccessManager *networkAccessManager();
+    QSharedPointer<QNetworkAccessManager> sharedNetworkAccessManager();
 
     /// Called by network jobs on credential errors, emits invalidCredentials()
     void handleInvalidCredentials();
 
+public slots:
+    /// Used when forgetting credentials
+    void clearQNAMCache();
+    void slotHandleSslErrors(QNetworkReply *, QList<QSslError>);
+
 signals:
     /// Emitted whenever there's network activity
     void propagatorNetworkActivity();
@@ -184,31 +245,46 @@ signals:
     /// Triggered by handleInvalidCredentials()
     void invalidCredentials();
 
-    void credentialsFetched(AbstractCredentials* credentials);
-    void credentialsAsked(AbstractCredentials* credentials);
+    void credentialsFetched(AbstractCredentials *credentials);
+    void credentialsAsked(AbstractCredentials *credentials);
 
     /// Forwards from QNetworkAccessManager::proxyAuthenticationRequired().
-    void proxyAuthenticationRequired(const QNetworkProxy&, QAuthenticator*);
+    void proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *);
 
     // e.g. when the approved SSL certificates changed
-    void wantsAccountSaved(Account* acc);
+    void wantsAccountSaved(Account *acc);
 
-    void serverVersionChanged(Account* account, const QString& newVersion, const QString& oldVersion);
+    void serverVersionChanged(Account *account, const QString &newVersion, const QString &oldVersion);
+
+    void accountChangedAvatar();
+    void accountChangedDisplayName();
 
 protected Q_SLOTS:
-    void slotHandleSslErrors(QNetworkReply*,QList<QSslError>);
     void slotCredentialsFetched();
     void slotCredentialsAsked();
 
 private:
-    Account(QObject *parent = 0);
+    Account(QObject *parent = nullptr);
     void setSharedThis(AccountPtr sharedThis);
 
     QWeakPointer<Account> _sharedThis;
     QString _id;
     QString _davUser;
+    QString _displayName;
+#ifndef TOKEN_AUTH_ONLY
+    QPixmap _avatarImg;
+#endif
     QMap<QString, QVariant> _settingsMap;
     QUrl _url;
+
+    /** If url to use for any user-visible urls.
+     *
+     * If the server configures overwritehost this can be different from
+     * the connection url in _url. We retrieve the visible host through
+     * the ocs/v1.php/config endpoint in ConnectionValidator.
+     */
+    QUrl _userVisibleUrl;
+
     QList<QSslCertificate> _approvedCerts;
     QSslConfiguration _sslConfiguration;
     Capabilities _capabilities;
@@ -216,7 +292,8 @@ private:
     QScopedPointer<AbstractSslErrorHandler> _sslErrorHandler;
     QuotaInfo *_quotaInfo;
     QSharedPointer<QNetworkAccessManager> _am;
-    QSharedPointer<AbstractCredentials> _credentials;
+    QScopedPointer<AbstractCredentials> _credentials;
+    bool _http2Supported = false;
 
     /// Certificates that were explicitly rejected by the user
     QList<QSslCertificate> _rejectedCertificates;
@@ -226,7 +303,6 @@ private:
     QString _davPath; // defaults to value from theme, might be overwritten in brandings
     friend class AccountManager;
 };
-
 }
 
 Q_DECLARE_METATYPE(OCC::AccountPtr)
