@@ -63,7 +63,7 @@ public:
          * Emitted once when done
          *
          * Except when SyncEngine jumps directly to finalize() without going
-         * through slotFinished().
+         * through slotPropagationFinished().
          */
         Done
     };
@@ -91,22 +91,39 @@ public:
      */
     void adjustTotalsForFile(const SyncFileItem &item);
 
-    quint64 totalFiles() const;
-    quint64 completedFiles() const;
+    /**
+     * Update totals for item.
+     *
+     * adjustTotalsForFile() is called during the treewalk phase to collect
+     * the initial total size and file count.
+     * This function is called at most once per item during propagation
+     * to adjust them when its actual size has been determined.
+     *
+     * The value in item.size must be the same as during the call to
+     * adjustTotalsForFile() while newSize is the newly determined actual
+     * size.
+     */
+    void updateTotalsForFile(const SyncFileItem &item, qint64 newSize);
 
-    quint64 totalSize() const;
-    quint64 completedSize() const;
+    qint64 totalFiles() const;
+    qint64 completedFiles() const;
+
+    qint64 totalSize() const;
+    qint64 completedSize() const;
 
     /** Number of a file that is currently in progress. */
-    quint64 currentFile() const;
+    qint64 currentFile() const;
 
     /** Return true if the size needs to be taken in account in the total amount of time */
     static inline bool isSizeDependent(const SyncFileItem &item)
     {
-        return !item.isDirectory() && (item._instruction == CSYNC_INSTRUCTION_CONFLICT
-                                         || item._instruction == CSYNC_INSTRUCTION_SYNC
-                                         || item._instruction == CSYNC_INSTRUCTION_NEW
-                                         || item._instruction == CSYNC_INSTRUCTION_TYPE_CHANGE);
+        return !item.isDirectory()
+            && (item._instruction == CSYNC_INSTRUCTION_CONFLICT
+                || item._instruction == CSYNC_INSTRUCTION_SYNC
+                || item._instruction == CSYNC_INSTRUCTION_NEW
+                || item._instruction == CSYNC_INSTRUCTION_TYPE_CHANGE)
+            && !(item._type == ItemTypeVirtualFile
+                 || item._type == ItemTypeVirtualFileDehydration);
     }
 
     /**
@@ -115,7 +132,7 @@ public:
     struct Estimates
     {
         /// Estimated completion amount per second. (of bytes or files)
-        quint64 estimatedBandwidth;
+        qint64 estimatedBandwidth;
 
         /// Estimated time remaining in milliseconds.
         quint64 estimatedEta;
@@ -139,8 +156,8 @@ public:
         /** Returns the estimates about progress per second and eta. */
         Estimates estimates() const;
 
-        quint64 completed() const;
-        quint64 remaining() const;
+        qint64 completed() const;
+        qint64 remaining() const;
 
     private:
         /**
@@ -152,19 +169,19 @@ public:
          * Changes the _completed value and does sanity checks on
          * _prevCompleted and _total.
          */
-        void setCompleted(quint64 completed);
+        void setCompleted(qint64 completed);
 
         // Updated by update()
         double _progressPerSec;
-        quint64 _prevCompleted;
+        qint64 _prevCompleted;
 
         // Used to get to a good value faster when
         // progress measurement stats. See update().
         double _initialSmoothing;
 
         // Set and updated by ProgressInfo
-        quint64 _completed;
-        quint64 _total;
+        qint64 _completed;
+        qint64 _total;
 
         friend class ProgressInfo;
     };
@@ -181,11 +198,12 @@ public:
     SyncFileItem _lastCompletedItem;
 
     // Used during local and remote update phase
-    QString _currentDiscoveredFolder;
+    QString _currentDiscoveredRemoteFolder;
+    QString _currentDiscoveredLocalFolder;
 
     void setProgressComplete(const SyncFileItem &item);
 
-    void setProgressItem(const SyncFileItem &item, quint64 completed);
+    void setProgressItem(const SyncFileItem &item, qint64 completed);
 
     /**
      * Get the total completion estimate
@@ -232,7 +250,7 @@ private:
     Progress _fileProgress;
 
     // All size from completed jobs only.
-    quint64 _totalSizeOfCompletedJobs;
+    qint64 _totalSizeOfCompletedJobs;
 
     // The fastest observed rate of files per second in this sync.
     double _maxFilesPerSecond;
@@ -270,11 +288,9 @@ enum class ErrorCategory {
 class OWNCLOUDSYNC_EXPORT ProgressDispatcher : public QObject
 {
     Q_OBJECT
-
-    friend class Folder; // only allow Folder class to access the setting slots.
 public:
     static ProgressDispatcher *instance();
-    ~ProgressDispatcher();
+    ~ProgressDispatcher() override;
 
 signals:
     /**
@@ -295,11 +311,13 @@ signals:
      */
     void syncError(const QString &folder, const QString &message, ErrorCategory category);
 
-protected:
-    void setProgressInfo(const QString &folder, const ProgressInfo &progress);
+    /**
+     * @brief Emitted for a folder when a sync is done, listing all pending conflicts
+     */
+    void folderConflicts(const QString &folder, const QStringList &conflictPaths);
 
 private:
-    ProgressDispatcher(QObject *parent = 0);
+    ProgressDispatcher(QObject *parent = nullptr);
 
     QElapsedTimer _timer;
     static ProgressDispatcher *_instance;
